import requests
import lnpay_py
import json


def get_request(location):
    """
    Network utility method for making a GET call to a LNPay endpoint

    Parameters
    ----------
    location (str): URL path requested

    Returns
    -------
    Network response as a JSON Object.
    """
    endpoint = lnpay_py.__ENDPOINT_URL__ + location
    headers = {
        'X-Api-Key': lnpay_py.__PUBLIC_API_KEY__,
        'X-LNPay-sdk': lnpay_py.__VERSION__
    }

    r = requests.get(url=endpoint, headers=headers)
    return r.json()


def post_request(location, params):
    """
    Network utility method for making a POST call to a LNPay endpoint

    Parameters
    ----------
    location (str): URL path requested
    params (object): the `data` to be POSTed in the network request

    Returns
    -------
    Network response as a JSON Object.
    """
    endpoint = lnpay_py.__ENDPOINT_URL__ + location
    headers = {
        'Content-Type': 'application/json',
        'X-Api-Key': lnpay_py.__PUBLIC_API_KEY__,
        'X-LNPay-sdk': lnpay_py.__VERSION__
    }
    data = json.dumps(params)

    r = requests.post(url=endpoint, data=data, headers=headers)
    return r.json()



from .utility_helpers import get_request


class LNPayLnTx:
    """
    A class used for LnTx API Functions

    Learn more at https://docs.lnpay.co
    """

    def __init__(self, tx_id):
        """
        Parameters
        ----------
        tx_id (str) : The LNPay transaction id (ex. lntx_tVompfuizfryXznKhc38J8)
        """

        self.tx_id = tx_id

    def get_info(self):
        """
        Gets the invoice information for the `tx_id` set in the `__init__` of this class.

        Returns
        -------
        LnTx (Lightning Invoice) Object
        https://docs.lnpay.co/lntx/

        Example:
        ```
        {
          "id": "lntx_IaWG3yS6FB3ZQJDRjXkkkkk",
          "created_at": 1577600922,
          "updated_at": 1577600922,
          "dest_pubkey": "033868c219bdb51a33560d854d500fe7d3898a1ad9e05dd89d0007e11313588500",
          "payment_request": "lnbc10n1pw7l9l3pp5pr2sr8pdt2yjm04am0wktr7cphkt8c3gtlvq8qlgaqw9jsh3d6qsdpj2pshjampd3kzunrfde4jq3npw43k2ap6ypgx7mmjgesh2cm9wscqzpgxqrrss4z9zac6gtpskjkhtdsxjd0m4are599k4ya9al0ktqqf7y70xqt5xs3as53va424nsh2dxumdln3ymm2048550hrj5sw8nw2ajzc9p9l3s97",
          "r_hash_decoded": "f646ff04116d16b7a70e953034b1c7c475c771847eb43e66892039ff5589863b",
          "description_hash": null,
          "memo": "This is a memo",
          "num_satoshis": 11,
          "expiry": 86400,
          "expires_at": 1578992143,
          "payment_preimage": "71566c546f304e70507a486b7561356177426a7535716345713670482f45582b4e4262337a654mmmmm6d673d",
          "settled": 1,
          "settled_at": 1575992143
        }
        ```
        """
        return get_request('lntx/{}'.format(self.tx_id))



from .utility_helpers import post_request

__version__ = '0.1.0'

__VERSION__ = 'py' + __version__
__ENDPOINT_URL__ = 'https://lnpay.co/v1/'
__DEFAULT_WAK__ = ''
__PUBLIC_API_KEY__ = ''


def initialize(public_api_key, default_wak=None, params=None):
    """
    LNPay module initialization function required for interacting with the LNPay API.

    Parameters
    ----------
    public_api_key (str): Account public key from https://lnpay.co/dashboard/developers
    default_wak (str, optional): Default Wallet Access Key to use for a specific wallet when creating a `LNPayWallet`.
    params (Object): Object representing additional parameters to set globally. Example: {'endpoint_url': 'https://lnpay.co/v1/'}
    """

    if params is None:
        params = {}

    print('initializing lnpay..')

    global __VERSION__
    global __PUBLIC_API_KEY__
    global __ENDPOINT_URL__
    global __DEFAULT_WAK__

    __VERSION__ = 'py' + __version__
    __PUBLIC_API_KEY__ = public_api_key
    __ENDPOINT_URL__ = params.get('endpoint_url', __ENDPOINT_URL__)
    __DEFAULT_WAK__ = default_wak

def create_wallet(params):
    return post_request('wallet', params)



import lnpay_py
from .utility_helpers import get_request, post_request


class LNPayWallet:
    """
      Class for LNPay Wallet API Functions

      Learn more at docs.lnpay.co
    """

    def __init__(self, access_key_str):
        """
        Parameters
        ----------
        access_key_str (str): Access key for a specific wallet. Optional if `default_wak` is initialized in `LNPay`.
        """

        if access_key_str:
            self.access_key = access_key_str
        elif lnpay_py.__DEFAULT_WAK__ is not None:
            self.access_key = lnpay_py.__DEFAULT_WAK__
        else:
            raise AttributeError('No wallet access key specified! Set a default, or initialize LNPayWallet with one')

    def get_info(self):
        """
        Get the wallet object which includes current balance.

        https://docs.lnpay.co/wallet/get-balance

        Returns
        -------
        JSON Object representing the response of the LNPay API

        Example:
        ```
        {
          "id": "w_pXHvKoKeKfrv",
          "created_at": 1577600922,
          "updated_at": 1577600922,
          "user_label": "Wallet for ATM",
          "balance": 1000,
          "statusType": {
                "id": 200,
                "type": "wallet",
                "name": "active",
                "display_name": "Active"
            },
        }
        ```
        """
        return get_request('wallet/{}'.format(self.access_key))

    def get_transactions(self):
        """
        Get a list of wallet transactions that have been SETTLED.
        This includes only transactions that have an impact on wallet balance.
        These DO NOT include unsettled/unpaid invoices.

        https://docs.lnpay.co/wallet/get-transactions

        Returns
        -------
        List of wallet transactions.
        Example:
        ```
        [
            {
                "id": "wtx_OXWNFNYYY0NsGLwLx7AtcPM",
                "created_at": 1580475159,
                "wallet_id": "w_ALMWRQrSoSf4Qh",
                "num_satoshis": 158,
                "user_label": "asdf",
                "lnTx": {
                    "id": "lntx_hlLn7xArPmew8KKSjqJqy6Vo",
                    "created_at": 1580129586,
                    "updated_at": 1580129586,
                    "dest_pubkey": "033868c219bdb51a33560d854d500fe7d3898a1ad9e05dd89d0007e11313588500",
                    "payment_request": "",
                    "r_hash_decoded": "7e13af679b551d8b0b804c3f0c74b43352c2d4c332ead6d693a8ea0aa6b6beec",
                    "memo": "",
                    "description_hash": "",
                    "num_satoshis": 2,
                    "expiry": 0,
                    "expires_at": null,
                    "payment_preimage": "323550597244415037724d6a436765567a72735875667438626e66754d385547",
                    "settled": 1,
                    "settled_at": 1580129586
                },
                "wtxType": {
                    "id": 30,
                    "layer": "ln",
                    "name": "ln_transfer_in",
                    "display_name": "Transfer In"
                }
            }
        ]
        ```
        """
        return get_request('wallet/{}/transactions'.format(self.access_key))

    def create_invoice(self, params):
        """
        Generates an invoice for this wallet.

        https://docs.lnpay.co/wallet/generate-invoice

        Parameters
        ----------
        params (Object): Object representing an invoice request. Example: `{'num_satoshis': 2,'memo': 'Tester'}`

        Returns
        -------
        LnTx Object (https://docs.lnpay.co/lntx/)
        Example:
        ```
        {
          "id": "lntx_IaWG3yS6FB3ZQJDRjXkkkkk",
          "created_at": 1577600922,
          "updated_at": 1577600922,
          "dest_pubkey": "033868c219bdb51a33560d854d500fe7d3898a1ad9e05dd89d0007e11313588500",
          "payment_request": "lnbc10n1pw7l9l3pp5pr2sr8pdt2yjm04am0wktr7cphkt8c3gtlvq8qlgaqw9jsh3d6qsdpj2pshjampd3kzunrfde4jq3npw43k2ap6ypgx7mmjgesh2cm9wscqzpgxqrrss4z9zac6gtpskjkhtdsxjd0m4are599k4ya9al0ktqqf7y70xqt5xs3as53va424nsh2dxumdln3ymm2048550hrj5sw8nw2ajzc9p9l3s97",
          "r_hash_decoded": "f646ff04116d16b7a70e953034b1c7c475c771847eb43e66892039ff5589863b",
          "description_hash": null,
          "memo": "This is a memo",
          "num_satoshis": 11,
          "expiry": 86400,
          "expires_at": 1578992143,
          "payment_preimage": "71566c546f304e70507a486b7561356177426a7535716345713670482f45582b4e4262337a654mmmmm6d673d",
          "settled": 1,
          "settled_at": 1575992143
        }
        ```
        """
        return post_request('wallet/{}/invoice'.format(self.access_key), params)

    def pay_invoice(self, params):
        """
        Pay a LN invoice from the specified wallet.

        https://docs.lnpay.co/wallet/pay-invoice

        Parameters
        ----------
        params (Object): Object representing an invoice payment request. Example: `{'payment_request': 'ln....'}`

        Returns
        -------
        Returns invoice payment information if successful or a specific error directly from the Lightning Node.

        Successful example:
        ```
        {
            "id":"wtx_JOhQFFI826owtE51AfFC975c",
            "created_at":1577657602,
            "wallet_id":"w_hkjS9r6mTYeABc",
            "wtxType": {
                "id": 20,
                "layer": "ln",
                "name": "ln_withdrawal",
                "display_name": "LN Withdrawal"
            },
            "num_satoshis":-5,
            "user_label":"Test invoice for docs",
            "lnTx":{
                "id":"lntx_82yveCX2Wn0EkkdyzvyBv",
                "created_at":1577657602,
                "updated_at":1577657602,
                "dest_pubkey":"02c16cca44562b590dd279c942200bdccfd4f990c3a69fad620c10ef2f8228eaff",
                "payment_request":"lnbc50n1p0qjf84pp5f70yqjjvu0z0esf7hksnca44d8j440mk5743qv70ku6jy9ewj8eqdpz23jhxapqd9h8vmmfvdjjqen0wgsxgmmrwvxqyz5vqcqzyssp583w0tugt4scyek2dat72p389lau0j8u9t5qnep29y0c32hyfn8rqrzjqt0pr36g7ke9elfvaqq3wmfey6laun0z8v0lg0nf9fdhdncxsp0y5zxkp5qqnsgqqqqqqquyqqqqqksqrc9qy9qsqzmvy83s8np7yrlqs98ge90tj3wwhfawjtq3cewv4vavmq0p5c4anhkm2aeyzjcvycttfgzwtak7nrrk6e3m3td8g4t8ha06uzzare4cqqne839",
                "r_hash_decoded":"4f9e404a4ce3c4fcc13ebda13c76b569e55abf76a7ab1033cfb73522172e91f2",
                "memo":"Test invoice for docs",
                "num_satoshis":5,
                "expiry":86400,
                "expires_at":1577744002,
                "payment_preimage":"6631394e526f35325135563854574f316651513330527a6e4e47315630795147662f7066466e4276664a773d",
                "settled":1,
                "settled_at":1577657602
            }
        }
        ```

        Failure example:
        ```
        {
            "name":"Bad Request",
            "message":"invoice is already paid",
            "code":0,
            "status":400
        }
        ```
        """
        return post_request('wallet/{}/withdraw'.format(self.access_key), params)

    def internal_transfer(self, params):
        """
        Transfer satoshis from source wallet to destination wallet.

        https://docs.lnpay.co/wallet/transfers-between-wallets

        Parameters
        ----------
        params (Object): Object representing a transfer request. Example: `{'dest_wallet_id': 'w_XXX','num_satoshis': 1,'memo': 'Memo'}`

        Returns
        -------
        Transfer execution information.

        Successful example:
        ```
        {
            "wtx_transfer_in":{
                "id":"wtx_aM9r4YEUrwohStWaxW7lDpRs",
                "created_at":1579269413,
                "wallet_id":"w_n743yizWqe43Oz",
                "num_satoshis":1,
                "user_label":"Test transfer",
                "lnTx":null,
                "wtxType":{
                    "id":30,
                    "layer":"ln",
                    "name":"ln_transfer_in",
                    "display_name":"Transfer In"
                }
            }
        },
            "wtx_transfer_out":{
                "id":"wtx_S9okrFLSS9LR97fWkdDfJ6U",
                "created_at":1579269412,
                "wallet_id":"w_hkjS9r6mTYeABc",
                "num_satoshis":-1,
                "user_label":"Test transfer",
                "lnTx":null,
                "wtxType":{
                    "id":40,
                    "layer":"ln",
                    "name":"ln_transfer_out",
                    "display_name":"Transfer Out"
                }
            }
        }
        ```

        Failure example:
        ```
        {
            "name":"Bad Request",
            "message":"insufficient funds",
            "code":0,
            "status":400
        }
        ```
        """
        return post_request('wallet/{}/transfer'.format(self.access_key), params)

    def get_lnurl(self, params):
        """
        Generate an LNURL-withdraw link.

        Note: These LNURLs are ONE-TIME use. This is to prevent repeated access to the wallet.

        https://docs.lnpay.co/wallet/lnurl-withdraw

        Parameters
        ----------
        params (Object): Object representing a lnurl withdraw request. Example: `{'num_satoshis': 1,'memo': 'SatsBack'}`

        Returns
        -------
        Generated lnurl object.

        Example:
        ```
        {
            "lnurl":"LNURL1DP68GURN8GHJ7MRWWPSHJTNRDUHHVVF0W4EK2U30WASKCMR9WSHHWC2LFACXUM35DDR5736ZF4HXVS6VGEV8GU6YDE49GC30D3H82UNV94C8YMMRV4EHX0M0W36R66MGD95KS4JGFADRS4ZRFEXK2SN2FFUXUSMHFA98XDZ8D3T9SDECWVHR43"
        }
        ```

        """
        return get_request('wallet/{}/lnurl/withdraw?num_satoshis={}'.format(self.access_key, params['num_satoshis']))



import os
from setuptools import setup
from setuptools import find_packages
from lnpay_py import __version__

here = os.path.abspath(os.path.dirname(__file__))

with open(os.path.join(here, 'README.md')) as fp:
    long_description = fp.read()

with open('requirements.txt') as f:
    install_reqs = f.readlines()
    reqs = [str(ir) for ir in install_reqs]

setup(
    name='lnpay-py',
    version=__version__,
    packages=find_packages(),
    install_requires=reqs,
    url='https://github.com/lnpay/lnpay-py',
    license='MIT',
    author='LNPay',
    author_email='bootstrapbandit7@gmail.com',
    description='LNPay Python SDK - at the moment a basic wrapper for the LNPay API',
    long_description=long_description,
    long_description_content_type='text/markdown',
    include_package_data=True
)
